#!/usr/bin/env python

"""
CourseBuilder 

Coursebuilder is a preprocessor tool for [pandoc](https://pandoc.org) 
to generate multi-lingual curricula documentation tables from 
structured representations as a flatfile database. Data scheme and 
actual values are kept in YAML files in order to version them with git.

"""

from argparse import ArgumentParser
import itertools
import yaml
import textwrap
import string



class CourseBuilder:

    def __init__(self) -> None:
        self.__schema = None

    
    def set_schema(self,schema = None):
        self.__schema = schema

    def get_template(self,field,lang='de'):
        if hasattr(self.__schema[field],'template'):
            return self.__schema[field]['template'][lang]
        else:
            return "$value"        
        

    def is_translatable(self,field):
        if hasattr(self.__schema[field],'translatable'):
            return self.__schema[field]['translatable']
        else:
            return True        

    def generate_markdown(self,ti,pagebreak = False) -> str:

        line_length = 128
        column_ratio= 0.28

        h_len = int(line_length * column_ratio)
        d_len = line_length-h_len

        # if title:
        #     print('#',desc[lang]['name'],'\n')

        print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))

        headline = False

        for k,v in ti:

            h =  textwrap.wrap(k, h_len, break_long_words=False)
            wrapper = textwrap.TextWrapper(d_len)
            t = [wrapper.wrap(i) for i in v.split('\n') if i != '']
            t = list(itertools.chain.from_iterable(t))
                  
            # get rows
            rows = list(itertools.zip_longest(h,t,fillvalue=""))

            # expand rows
            for r in rows:
                print(''.join(['|',r[0].ljust(h_len,' '),'|',r[1].ljust(d_len,' '),'|']))

            if headline:
                print(''.join(['+',"".ljust(h_len,'-'),'+',"".ljust(d_len,'-'),'+']))
            else:
                print(''.join(['+',"".ljust(h_len,'='),'+',"".ljust(d_len,'='),'+']))
                headline = True

        # to control pagebreaks for pandoc
        if pagebreak:
            print('\n\\newpage')
        
    def process_label(self,field,lang='de'):
        # processes the label of a field item
        return self.__schema[field]['label'][lang]

    def process_str(self,meta,field,lang='de'):
        #  
        return [self.process_label(field,lang),meta[field][lang]]

    def process_enum(self,meta,field,lang='de'):
        v = meta[field]['value']
        return [self.process_label(field,lang),self.__schema[field]['values'][v][lang]]

    def process_int(self,meta,field,lang='de'):
        v = meta[field]['value']
        t = string.Template(self.get_template(field,lang))
        return [self.process_label(field,lang),t.substitute({'value' : v})]

    def process(self,meta,fields = [],lang = 'de'):

        table_items = []

        for field in fields:
            match self.__schema[field]['type']: 
                case 'str': table_items.append(self.process_str(meta,field,lang)) 
                case 'enum': table_items.append(self.process_enum(meta,field,lang))
                case 'int': table_items.append(self.process_int(meta,field,lang))
        
        self.generate_markdown(table_items)


def main():
    
    # get command line parameters
    parser = ArgumentParser()

    # arguments
    parser = ArgumentParser(description='versatile curricula generator')

    parser.add_argument('-m','--meta',action="extend", nargs="+", type=str,help="course description(s) as YAML file(s)")
    parser.add_argument('-l','--lang',help="Language to parse from meta file (use de or en)",default='de')
    parser.add_argument('-f','--fields',help="Fields to be used",action="extend", nargs="+", type=str)
    
    parser.add_argument('-s','--schema',help="using provided schema")
    
    
    # get arguments
    args = parser.parse_args()


    # only run debug
    if args.schema:
        
        cb = CourseBuilder()

        with open(args.schema) as f:
            cb.set_schema(yaml.load(f,Loader=yaml.Loader))

        for m in args.meta:
            with open(m) as fm:
                cb.process(yaml.load(fm,Loader=yaml.Loader),fields=args.fields,lang=args.lang)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()
